import { Component, OnInit, Input, ViewChild, ElementRef, ViewEncapsulation } from '@angular/core';
import { Browser } from '@syncfusion/ej2-base';
import { Observable } from 'rxjs';
import { EditService, ToolbarService, PageService, DialogEditEventArgs, SaveEventArgs, DataSourceChangedEventArgs, DataStateChangeEventArgs, GridComponent } from '@syncfusion/ej2-angular-grids';
import { DataUtil } from '@syncfusion/ej2-data';
import { FormGroup, AbstractControl, FormControl, Validators } from '@angular/forms';
import { ComboBoxComponent, ChangeEventArgs } from '@syncfusion/ej2-angular-dropdowns'
import { Dialog, AnimationSettingsModel, DialogComponent } from '@syncfusion/ej2-angular-popups';
import { IFollowerType } from './../../models/lookup.model';
import { IGroupMember, IGroupMemberContact } from './../../models/group.model';
import { LookupService } from './../../services/lookupServices';
import { GroupMemberService } from './../../services/groupMember.service';

@Component({
  selector: 'app-group-members',
  templateUrl: './group-members.component.html',
    styleUrls: ['./group-members.component.css'],
    providers: [ToolbarService, EditService, PageService, LookupService, GroupMemberService],
    encapsulation: ViewEncapsulation.None
})
export class GroupMembersComponent implements OnInit {
    public data: Observable<DataStateChangeEventArgs>;
    public editSettings: Object;
    public toolbar: string[];
    public pageSettings: Object;
    public submitClicked: boolean = false;
    public groupMemberForm: FormGroup;
    public followerTypeDataSet: IFollowerType[];
    public isEditing: Boolean = false;
    public followerTypeFields: Object = {
        value: 'Id', text: 'FollowerTypeName'
    };
    public followerTypePlaceHolder: string = "Follower Type";

    public genderDataSet: Object[] = [{
        id: 'M', name: 'Male'
    }, {
        id: 'F', name: 'Female'
    }];


    public genderFields: Object = {
        value: 'id', text: 'name'
    };
    public genderPlaceHolder: string = "Gender";


    private _groupId: string = "";
    @Input()
    set groupId(groupID: string) {
        this._groupId = groupID;
        this.getData();
    };
    get groupId(): string { return this._groupId; }

    @ViewChild('groupMemberGrid', { static: false })
    public groupMemberGridObj: GridComponent;

    public target: string = '#targetModal';
    public width: string = '335px';
    public isModal: Boolean = true;
    public content: string = "This is testing";
    public animationSettings: AnimationSettingsModel = { effect: 'None' };

    @ViewChild('closeButton', {static: false})
    public closeButton: ElementRef;

    @ViewChild('modalContactsDialog', { static: false })
    public modalContactsDialog: DialogComponent;

    constructor(private lookupService: LookupService, private groupMemberService: GroupMemberService) {
        this.data = groupMemberService;
    }

    createFormGroup(data: IGroupMember): FormGroup {
        //console.log("Reached");
        if (data.Id !== null && data.Id !== undefined) {

            return new FormGroup({
                Id: new FormControl(data.Id),
                GroupId: new FormControl(data.GroupId),
                FirstName: new FormControl(data.FirstName, Validators.required),
                LastName: new FormControl(data.LastName, Validators.required),
                JerseyNumber: new FormControl(data.JerseyNumber),
                Position: new FormControl(data.Position),
                Gender: new FormControl(data.Gender),
                BirthDate: new FormControl(data.BirthDate),
                ActiveParticipant: new FormControl(data.ActiveParticipant),
                FollowerTypeId: new FormControl(data.FollowerTypeId),
                ContactFirstName: new FormControl(data.ContactFirstName),
                ContactLastName: new FormControl(data.ContactLastName),
                ContactEmailAddress: new FormControl(data.ContactEmailAddress, Validators.email),
                CreatedBy: new FormControl(data.CreatedBy),
                CreatedDate: new FormControl(data.CreatedDate),
                ModifiedBy: new FormControl(data.ModifiedBy),
                ModifiedDate: new FormControl(data.ModifiedDate),
                GroupMemberContact: new FormControl(data.GroupMemberContact)

            });
        }
        else {
            
            //console.log(data.GroupMemberContact);
            return new FormGroup({

                GroupId: new FormControl(data.GroupId),
                FirstName: new FormControl(data.FirstName, Validators.required),
                LastName: new FormControl(data.LastName, Validators.required),
                JerseyNumber: new FormControl(data.JerseyNumber),
                Position: new FormControl(data.Position),
                Gender: new FormControl(data.Gender),
                BirthDate: new FormControl(data.BirthDate),
                ActiveParticipant: new FormControl(data.ActiveParticipant),
                FollowerTypeId: new FormControl(data.FollowerTypeId),
                ContactFirstName: new FormControl(data.ContactFirstName),
                ContactLastName: new FormControl(data.ContactLastName),
                ContactEmailAddress: new FormControl(data.ContactEmailAddress, Validators.email)
            });
        }
    }
    ngOnInit() {
        this.lookupService.getFollowerTypes().subscribe(res => { this.followerTypeDataSet = res; });
        this.editSettings = { allowEditing: true, allowAdding: true, allowDeleting: true, mode: 'Dialog' };
        this.toolbar = ['Add', 'Edit', 'Delete'];
        this.pageSettings = { pageCount: 20 };
        this.getData();

  }
    public getData() {
        const state: any = { skip: 0, take: 12 };
        //console.log(this.groupId);
        this.groupMemberService.executeGroupMembers(this.groupId, state);
    }
    public dataStateChange(state: DataStateChangeEventArgs): void {
        //this.getData();

        this.groupMemberService.executeGroupMembers(this.groupId, state);

    }

    public dataSourceChanged(state: DataSourceChangedEventArgs): void {

        if (state.action === 'add') {

            
            //state.endEdit();
            this.groupMemberService.addGroupMember(state).subscribe(() => { }, error => console.log(error), () => {
                state.endEdit();
            });
        } else if (state.action === 'edit') {

            this.groupMemberService.updateGroupMember(state).subscribe(() => {
                state.endEdit();
            }, (e) => {
                    this.groupMemberGridObj.closeEdit();
            }
            );
        } else if (state.requestType === 'delete') {
            this.groupMemberService.deleteGroupMember(state).subscribe(() => {
                state.endEdit();
            });
        }
    }


    actionBegin(args: SaveEventArgs): void {
        if (args.requestType === 'beginEdit' || args.requestType === 'add') {
            this.submitClicked = false;

            if (args.requestType === 'add') {
                (<IGroupMember>args.rowData).GroupId = this.groupId;
            }
            else {
                //console.log(args.rowData);
                this.isEditing = true;
            }

            
            this.groupMemberForm = this.createFormGroup(<IGroupMember>args.rowData);
        }
        if (args.requestType === 'save') {
            this.submitClicked = true;
            if (this.groupMemberForm.valid) {
                let formValues: IGroupMember = this.groupMemberForm.value;
                if (formValues.Id === undefined || formValues.Id === null) {
                    let groupMemberContacts: IGroupMemberContact[] = [];
                    let groupMemberContact: IGroupMemberContact = {
                        FirstName: formValues.ContactFirstName,
                        LastName: formValues.ContactLastName,
                        EmailAddress: formValues.ContactEmailAddress,
                        IsPrimary: true,
                        AcceptedGroupMemberContact: false
                    };
                    groupMemberContacts.push(groupMemberContact);
                    formValues.GroupMemberContact = groupMemberContacts;
                    args.data = formValues;
                    //(<IGroupMember>args.data).GroupMemberContact.push(groupMemberContact);
                }
                else {
                    console.log(formValues);
                    let selectedFollowerType: IFollowerType = this.followerTypeDataSet.filter(ft => ft.Id == formValues.FollowerTypeId)[0];
                    if (selectedFollowerType !== null)
                        formValues.FollowerTypeName = selectedFollowerType.FollowerTypeName;
                    formValues.DisplayName = `${formValues.FirstName} ${formValues.LastName}`;
                    
                    args.data = formValues;
                }
            } else {
                args.cancel = true;
            }
        }
    }

    actionComplete(args: DialogEditEventArgs): void {
        if ((args.requestType === 'beginEdit' || args.requestType === 'add')) {

            if (args.requestType === "add") {
                args.dialog.header = "Add new member";
            }
            else {
                args.dialog.header = `Edit member - ${(<IGroupMember>args.rowData).DisplayName}`;
            }
            if (Browser.isDevice) {
                args.dialog.height = window.innerHeight - 90 + 'px';
                (<Dialog>args.dialog).dataBind();
            }

            (args.form.elements.namedItem('FirstName') as HTMLInputElement).focus();

        }
    }
    public manageContacts(id: string) {
        this.modalContactsDialog.show();
    }

    public dialogClose = (): void => {
        //this.dlgbtn.element.style.display = '';
    }

    public dialogOpen = (): void => {
        //this.dlgbtn.element.style.display = 'none';
        //(document.getElementById('sendButton') as any).keypress = (e: any) => {
        //    if (e.keyCode === 13) { this.updateTextValue(); }
        //};
        //(document.getElementById('inVal') as HTMLElement).onkeydown = (e: any) => {
        //    if (e.keyCode === 13) { this.updateTextValue(); }
        //};
        //document.getElementById('sendButton').onclick = (): void => {
        //    this.updateTextValue();
        //};
    }
    get FirstName(): AbstractControl { return this.groupMemberForm.get('FirstName'); }
    get LastName(): AbstractControl { return this.groupMemberForm.get('LastName'); }
    get ContactEmailAddress(): AbstractControl { return this.groupMemberForm.get('ContactEmailAddress'); }
}
